import os

import pytest

from d8s_yaml import yaml_read, is_yaml, yaml_write, yaml_sort, yaml_files, yaml_clean
from d8s_file_system import directory_delete, directory_create, file_write

NON_EXISTENT_FILE_PATH = './foo'
TEST_DIRECTORY_PATH = './test_files'
TEST_FILE_CONTENTS = 'a'
TEST_FILE_NAME_A = 'a.yml'
TEST_FILE_PATH_A = os.path.join(TEST_DIRECTORY_PATH, TEST_FILE_NAME_A)
TEST_FILE_NAME_B = 'a.yaml'
TEST_FILE_PATH_B = os.path.join(TEST_DIRECTORY_PATH, TEST_FILE_NAME_B)


@pytest.fixture(autouse=True)
def clear_testing_directory():
    """This function is run after every test."""
    directory_delete(TEST_DIRECTORY_PATH)
    directory_create(TEST_DIRECTORY_PATH)
    file_write(TEST_FILE_PATH_A, TEST_FILE_CONTENTS)
    file_write(TEST_FILE_PATH_B, TEST_FILE_CONTENTS)


def setup_module():
    """This function is run before all of the tests in this file are run."""
    directory_create(TEST_DIRECTORY_PATH)


def teardown_module():
    """This function is run after all of the tests in this file are run."""
    directory_delete(TEST_DIRECTORY_PATH)


def test_yaml_files_1():
    result = yaml_files(TEST_DIRECTORY_PATH)
    assert result == ['a.yaml']

    result = yaml_files(TEST_DIRECTORY_PATH, include_yml_extensions=True)
    assert result == ['a.yaml', 'a.yml']


def test_yaml_clean_1():
    s = '''b:
  d: 2
  c: [3, 4]
a: Easy!'''
    result = yaml_clean(s)
    assert (
        result
        == '''a: Easy!
b:
  c:
  - 3
  - 4
  d: 2
'''
    )


def test_yaml_sort_1():
    s = '''b:
  d: 2
  c: [3, 4]
a: Easy!'''

    result = yaml_read(s)
    assert result == {'b': {'d': 2, 'c': [3, 4]}, 'a': 'Easy!'}

    result = yaml_sort(s)
    assert result == '''a: Easy!\nb:\n  c:\n  - 3\n  - 4\n  d: 2\n'''


def test_yaml_read_1():
    s = '''sunday: 10
monday: 11'''
    result = yaml_read(s)
    assert result == {'monday': 11, 'sunday': 10}

    s = '''
a: Easy!
b:
  c: 2
  d: [3, 4]
'''
    result = yaml_read(s)
    print(result)
    assert result == {'a': 'Easy!', 'b': {'c': 2, 'd': [3, 4]}}


def test_yaml_write_1():
    d = {'a': 'Easy!', 'b': {'c': 2, 'd': [3, 4]}}
    result = yaml_write(d)
    assert result == 'a: Easy!\nb:\n  c: 2\n  d:\n  - 3\n  - 4\n'


def test_is_yaml_1():
    s = '''sunday: 10
monday: 11'''
    assert is_yaml(s)

    s = '''foo:: bar
bing buzz boo'''
    assert not is_yaml(s)



from typing import List, Dict, Any, Union

Json = Union[List[Any], Dict[Any, Any], str]

# TODO: i think it may be more helpful to name functions like `yaml_read` as `yaml_from_string` or `yaml_to_json`


def yaml_files(path, *, include_yml_extensions: bool = False):
    """."""
    from d8s_file_system import directory_file_names_matching

    # TODO (oct 2020): in this example, we are potentially running the directory_file_names_matching function twice - I'm ok with this for now, but would like to consider caching results

    pattern = '*.yaml'
    files = directory_file_names_matching(path, pattern)

    if include_yml_extensions:
        yml_pattern = '*.yml'
        files.extend(directory_file_names_matching(path, yml_pattern))

    return files


def yaml_read(yaml_data: str):
    import yaml

    parsed_yaml_data = yaml.load(yaml_data, Loader=yaml.Loader)
    return parsed_yaml_data


def is_yaml(possible_yaml_data: str) -> bool:
    try:
        yaml_read(possible_yaml_data)
    except:
        return False
    else:
        return True


# todo: are there other input types possible?
def yaml_write(data: Json, **kwargs) -> str:
    import yaml

    result = yaml.dump(data, **kwargs)
    return result


# todo: are there other input types possible?
def yaml_clean(yaml_data: str) -> str:
    """Standardize the given yaml data."""
    return yaml_standardize(yaml_data)


# todo: are there other input types possible?
def yaml_standardize(yaml_data: str) -> str:
    """Standardize the given yaml data by reading and writing it."""
    read_data = yaml_read(yaml_data)
    sorted_data = yaml_write(read_data)
    return sorted_data


# todo: are there other input types possible?
def yaml_sort(yaml_data: str) -> str:
    return yaml_standardize(yaml_data)
